/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

var cache = {
    ERROR_SQL: 1,
    ERROR_NOT_FOUND: 2,
    ERROR_AJAX: 3,

    /**
     * Wrapper function for executing ALL database queries
     * @param string sql
     * @param array params
    */
    executeSql: function(sql, params){
        if(typeof params === 'undefined'){
            params = [];
        }

        var $d = $.Deferred();

        app.db.executeSql(sql, params, function(results){
            $d.resolve(results);
        },
        function(error){
            $d.reject(cache.ERROR_SQL);
        });

        return $d.promise();
    },


    /*
     * Fetches a single row of results
     * @param string sql
     * @param array params
     *
     * @return Object
    */
    fetchRow: function(sql, params){
        var $d = $.Deferred();

        if(typeof params === 'undefined'){
            params = [];
        }

        cache.executeSql(sql, params).done(function(results){
            var ret = {};
            if(results.rows.length > 0){
                ret = results.rows.item(0);
            }

            $d.resolve(ret);
        });
        return $d.promise();
    },

    /*
     * Fetches a single column from the results
     * @param string sql
     * @param array params
     *
     * @return string|int
    */
    fetchColumn: function(sql, params){
        var $d = $.Deferred();

        if(typeof params === 'undefined'){
            params = [];
        }

        cache.executeSql(sql, params).done(function(results){
            var ret = '';
            if(results.rows.length > 0){
                var item = results.rows.item(0);
                var keys = Object.keys(item);

                if(keys.length > 0){
                    ret = item[keys[0]];
                }
            }
            $d.resolve(ret);
        });
        return $d.promise();
    },

    /*
     * Fetches an array of results
     * @param string sql
     * @param array params
     *
     * @param array
    */
    fetchAll: function(sql, params){
        var $d = $.Deferred();

        if(typeof params === 'undefined'){
            params = [];
        }

        cache.executeSql(sql, params).done(function(results){
            var ret = [];
            for(var i =0; i < results.rows.length; i++){
                var item = results.rows.item(i);
                ret.push(item);
            }
            $d.resolve(ret);
        });
        return $d.promise();
    },

    /**
     * Loads all of the data from a database table
     * @param string dbTable
     * @param array order
     * @param array joins
     * @param array wheres
    */
    loadTable: function(dbTable, order, joins, wheres){
        var $d = $.Deferred();

        var joinSql ='', whereSql='', orderSql='';

        //Joins
        if(joins instanceof Array && joins.length > 0){
            joinSql = ' ' + joins.join(' ');
        }
        //Where conditions
        if(wheres instanceof Array && wheres.length > 0){
            whereSql = ' WHERE ' + wheres.join(' AND ');
        }
        //Order by
        if(order instanceof Array && order.length > 0){
            orderSql = ' ORDER BY ' + order.join(', ');
        }

        var sql = sprintf('SELECT * FROM %s %s %s %s', dbTable, joinSql, whereSql, orderSql);

        cache.fetchAll(sql, []).done(function(results){
            $d.resolve(results);
        });

        return $d.promise();
    },

    /**
     * Loads a single row from a database table based on the primary key id field
     * @param string dbTable
     * @param integer id
    */
    loadSingle: function(dbTable, id){
        var $d = $.Deferred();
        var sql = sprintf('SELECT * FROM %s WHERE id = ?', dbTable);

        cache.fetchRow(sql, [ id ]).done(function(row){
            $d.resolve(row);
        });
        return $d.promise();
    },

    /**
     * Inserts data into database table
     * @param string dbTable
     * @param array data
    */
    insertData: function(dbTable, data){
        var $d = $.Deferred();

        var columns = [];
        var values = [];
        var sqlParams = [];

        $.each(data, function(index, value){
            columns.push(index);
            values.push('?');
            sqlParams.push(value);
        });

        var insertSql = sprintf('INSERT INTO %s ( %s ) VALUES ( %s )', dbTable, columns.join(', '), values.join(' ,'));

        cache.executeSql(insertSql, sqlParams).done(function(){
            $d.resolve(app.db.lastInsertRowId);
        });

        return $d.promise();
    },

    /**
     * Removes a row a database table
     * @param string dbTable
     * @param string column
     * @param string value
    */
    deleteData: function(dbTable, column, value){
        var $d = $.Deferred();

        var deleteSql = sprintf('DELETE FROM %s WHERE %s = ?', dbTable, column);

        cache.executeSql(deleteSql, [value]).done(function(){
            $d.resolve();
        });

        return $d.promise();
    },

    /**
     * Updates data in database table
     *
     * @param string dbTable
     * @param array data
     * @param array where
    */
    updateData: function(dbTable, data, where){
        var $d = $.Deferred();

        var sqlParams = [];
        var whereParams = [];

        //columns to be updated
        var columns = [];
        $.each(data, function(index, value) {
            columns.push(index + '=?');
            sqlParams.push(value.toString());
        });

        //where clause
        var whereClauses = [];
        $.each(where, function(whereKey, whereValue){
            whereClauses.push(whereKey + '=?');
            whereParams.push(whereValue.toString());
            sqlParams.push(whereValue.toString());
        });

        var updateSql = sprintf('UPDATE %s SET %s WHERE %s', dbTable, columns.join(', '), whereClauses.join(' AND '));
        cache.executeSql(updateSql, sqlParams);

        //Return ID of row updated
        var findSql = sprintf('SELECT id FROM %s WHERE %s', dbTable, whereClauses.join(' AND '));
        cache.fetchColumn(findSql, whereParams).done(function(id){
            $d.resolve(id);
        });

        return $d.promise();
    },

    /*
     * Check if the data exists in the cache
     * @param string dbTable
     * @param string columnName
     * @param string columnValue
    */
    dataExists: function(dbTable, columnName, columnValue){
        var $d = $.Deferred();

        var sql = sprintf('SELECT count(*) AS count FROM %s WHERE %s = ?', dbTable, columnName);
        cache.fetchColumn(sql, [ columnValue ]).done(function (count) {
            $d.resolve(count > 0);
        });

        return $d.promise();
    },

    /*
     * Based on an array of values remove all entries from a table that match a particular unique column
     * @param string dbTable
     * @param string columnName
     * @param array values
    */
    cleanupData: function(dbTable, columnName, values){
        var $d = $.Deferred();

        if(values.length > 0){
            var placeholders = [];
            for(var i =0; i < values.length; i++){
                placeholders.push('?');
            }

            var sql = sprintf('DELETE FROM %s WHERE %s NOT IN ( %s )', dbTable, columnName, placeholders.join(', '));

            cache.executeSql(sql, values);
            $d.resolve();
        }
        else{
            $d.resolve();
        }

        return $d.promise();
    },

    /*
     * Creates a link between tables
     * @param string joinTable
     * @param string parentIdColumn,
     * @param string childIdColumn
     * @param string dbTable
     * @param string childTable
     * @param object parent
     * @param object child
    */
    createLink: function(joinTable, parentIdColumn, childIdColumn, dbTable, childTable, parent, child){
        var $d = $.Deferred();

        /*
            Example query:
                INSERT INTO events_times (event_id, time_id)
                SELECT 1, 5 WHERE NOT EXISTS(SELECT 1 FROM events_times WHERE event_id = 1 AND time_id = 5)
        */

        //Firstly get the parentId & childId
        var getParentId = function(){
            var sql = sprintf('SELECT id FROM %s WHERE recite_id = ?', dbTable);
            return cache.fetchColumn(sql, [parent.recite_id]);
        };
        var getChildId = function(){
            var sql = sprintf('SELECT id FROM %s WHERE recite_id = ?', childTable);
            return cache.fetchColumn(sql, [child.recite_id]);
        };

        $.when(
            getParentId(),
            getChildId()
        ).done(function(parentId, childId){
            var replacements = {
                "joinTable": joinTable,
                "parentIdColumn": parentIdColumn,
                "childIdColumn": childIdColumn,
                "parentId": parentId,
                "childId": childId
            };
            var sql = sprintf('INSERT INTO %(joinTable)s ( %(parentIdColumn)s, %(childIdColumn)s ) SELECT %(parentId)s, %(childId)s WHERE NOT EXISTS ( SELECT 1 FROM %(joinTable)s WHERE %(parentIdColumn)s = %(parentId)s AND %(childIdColumn)s = %(childId)s )', replacements);

            cache.executeSql(sql, []);
            $d.resolve();
        });

        return $d.promise();
    },

    /**
     * See if we need to check for an update
     *
     * @param string table
     * @param string column
     * @param string value
     *
     * @return
    */
    needToCheckForUpdates: function(table, column, value){
        var $d = $.Deferred();
        var nowTs = Math.floor(Date.now() / 1000);

        var sql = sprintf('SELECT * FROM %s WHERE %s = ?', table, column);

        cache.fetchRow(sql, [ value ]).done(function (row) {
            if(row.lastChecked.length > 0){
                if(row.lastChecked <= (nowTs - app.cacheTime)){
                    //Content has expired, check for an update
                    $d.resolve(true, row);
                }
                else{
                    //Recently checked for an update, no need to check for an update
                    $d.resolve(false);
                }
            }
            else{
                //Content not found in local database
                $d.reject(cache.ERROR_NOT_FOUND);
            }
        });

        return $d.promise();
    },

    /**
     * Downloads updates
     * @param string dataUrl
     * @param string lastUpdate
     * @param string reciteId (optional)
    */
    _downloadUpdates: function(dataUrl, lastUpdate, reciteId){
        var url = sprintf('%s?lastUpdate=%s', dataUrl, lastUpdate);

        if(typeof reciteId !== 'undefined'){
            url += sprintf('&recite_id=%s', reciteId);
        }

        var $d = $.getJSON(url);

        return $d;
    },

    _processUpdateRow: function(row, dbTable){
        var $d = $.Deferred();

        row.lastChecked = Math.floor(Date.now() / 1000); //now

        if(row.status === 'NOCHANGE'){
            $d.resolve();
        }
        else{
            //Clone object
            var columns = JSON.parse(JSON.stringify(row));
            //Ignore children
            if('children' in columns){
                delete columns.children;
            }

            //Firstly check if the entity already exists
            cache.dataExists(dbTable, 'recite_id', columns.recite_id).done(function(exists){
                //Update data in db
                if(exists){
                    cache.updateData(dbTable, columns, { recite_id: columns.recite_id }).done(function(id){
                        $d.resolve(id);
                    });
                }
                //Insert data into db
                else{
                    cache.insertData(dbTable, columns).done(function(id){
                        $d.resolve(id);
                    });
                }
            });
        }

        return $d.promise();
    },

    /**
     * Makes a JSON call to check for updates and download them
     * @param string dataUrl
     * @param string dbTable
     * @param string lastUpdate
     * @param integer recite_id
    */
    checkAndDownloadUpdatesSingleObject: function(dataUrl, dbTable, lastUpdate, recite_id){
        var $d = $.Deferred();

        cache._downloadUpdates(dataUrl, lastUpdate, recite_id).done(function(data){
            if(data.length > 0){
                var first = data[0];

                cache._processUpdateRow(first, dbTable).done(function(){
                    $d.resolve();
                }).fail(function(){
                    $d.reject();
                });
            }
        }).fail(function(){
            $d.reject(cache.ERROR_AJAX);
        });

        return $d.promise();
    },

    /**
     * Makes a JSON call to check for updates and download them
     * @param string dataUrl
     * @param string entity
     * @param string dbTable
     * @param string lastUpdate
     * @param string childTable (optional)
     * @param string joinTable (optional)
    */
    checkAndDownloadUpdates: function(dataUrl, entity, dbTable, lastUpdate, childTable, joinTable){
        var $deferredObject = $.Deferred();

        var nowTs = Math.floor(Date.now() / 1000);
        var parentColumn = dbTable.replace(/s$/, '') + '_id'; //follows particular naming scheme - <original_table(singular)>_id
        var childColumn = '';

        if(childTable){
            childColumn = childTable.replace(/s$/, '') + '_id'; //follows particular naming scheme - <original_table(singular)>_id
        }

        cache._downloadUpdates(dataUrl, lastUpdate).done(function(data){
            if(data.length > 0){
                var reciteIds = [];
                var childrenIds = [];
                var childTableSet = (typeof childTable !== 'undefined' && childTable !== null && childTable.length > 0);
                var nowTs = Math.floor(Date.now() / 1000);

                var processRow = function(row){
                    var $d = $.Deferred();

                    reciteIds.push(row.recite_id);

                    cache._processUpdateRow(row, dbTable).done(function(){
                        if(childTableSet && 'children' in row){
                            var promises = [];
                            $.each(row.children, function(key, child){
                                promises.push(
                                    processChildRow(child, row)
                                );
                            });

                            $.when.apply($, promises).done(function(){
                                $d.resolve();
                            });
                        }
                        else{
                            $d.resolve();
                        }
                    });

                    return $d.promise();
                };

                var processChildRow = function(child, parent){
                    var $d = $.Deferred();

                    childrenIds.push(child.recite_id);

                    cache._processUpdateRow(child, childTable).done(function(){
                        //Create link in join table
                        if(child.status !== 'NOCHANGE'){
                            cache.createLink(joinTable, parentColumn, childColumn, dbTable, childTable, parent, child).done(function(){
                                $d.resolve();
                            });
                        }
                        else{
                            $d.resolve();
                        }
                    });

                    return $d.promise();
                };

                var promises = [];

                $.each(data, function(i, row){
                    promises.push (
                        processRow(row)
                    );
                });

                $.when.apply($, promises).done(function(){
                    $.when(
                        cache.cleanupData(dbTable, 'recite_id', reciteIds),
                        cache.cleanupData(childTable, 'recite_id', childrenIds),
                        cache.updateData(app.checksTable, { lastChecked: nowTs } , { entity: entity })
                    ).done(function(){
                        $deferredObject.resolve();
                    });
                });
            }
        }).fail(function(){
            $deferredObject.reject(cache.ERROR_AJAX);
        });

        return $deferredObject.promise();
    }
};


var app = {
    cacheTime: 3600, //60 minutes
    delegatesLink: 'https://app.performingartsmarket.com.au/delegates',
    contentUrl: 'https://www.performingartsmarket.com.au/__/apam/contentFeed',
    venuesFeedUrl: 'https://www.performingartsmarket.com.au/__/apam/venuesFeed',
    programFeedUrl: 'https://www.performingartsmarket.com.au/__/apam/programFeed',
    newsFeedUrl: 'https://www.performingartsmarket.com.au/__/apam/newsFeed',
    notificationsFeedUrl: 'https://www.performingartsmarket.com.au/__/apam/notificationsFeed',
    adsFeedUrl: 'https://www.performingartsmarket.com.au/ads_feed.json',

    contentTable: 'content',
    venuesTable: 'venues',
    newsTable: 'news',
    notificationsTable: 'notifications',
    checksTable: 'checks',

    programTable: 'events',
    timesTable: 'times',
    programTimesJoinTable: 'events_times',

    contentTableColumns: [ 'title', 'content' ],
    venuesTableColumns: [ 'title', 'street_address', 'suburb', 'phone_number_australia', 'phone_number_international', 'thumbnail', 'image', 'map_image', 'google_maps_link', 'sort' ],

    venueId: '',
    eventId: '',
    pageTitle: '',
    db: null,

    rootScope: null,

    parseAppID: 'JHe1tuxaEIZk7UrOCAoql51GeNRnibce2IffrbXm',
    parseClientKey: 'zxbwWIKJb9UM4eRPSYiHkwsMokzpmmQxo9MGz2Ur',

    // Application Constructor
    initialize: function() {
        this.bindEvents();
    },
    // Bind Event Listeners
    //
    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
    bindEvents: function() {
        document.addEventListener('deviceready', this.onDeviceReady, false);
    },
    // deviceready Event Handler
    onDeviceReady: function() {
        app.db = window.sqlitePlugin.openDatabase({name: "apam2018_1.23.db", location: 2, createFromLocation: 1});

        var versionUpdates_1 = function(){
            var $d = $.Deferred();

            $.when(
                cache.executeSql('CREATE TABLE IF NOT EXISTS favourite_events (id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE, event_id INTEGER UNIQUE)'),
                cache.executeSql('CREATE TABLE IF NOT EXISTS event_notes (id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE, event_id INTEGER UNIQUE, note TEXT)'),
                cache.executeSql('CREATE TABLE IF NOT EXISTS notifications (id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE, recite_id INTEGER UNIQUE, title TEXT, lastUpdate TEXT, lastChecked TEXT, message TEXT)'),
                cache.executeSql('UPDATE events SET lastChecked = 0, lastUpdate = 0'), //Force update of events table (data changes to event_type column)
                cache.insertData('versions', { version: 1 })
            ).done(function(){
                //Insert record into checks for entity notifications
                cache.insertData('checks', { entity: 'notifications', lastChecked: Math.floor(Date.now() / 1000) });

                //Download notifications
                cache.checkAndDownloadUpdates(app.notificationsFeedUrl, 'notifications', app.notificationsTable, 0);

                //Download updates to events table
                cache.checkAndDownloadUpdates(app.programFeedUrl, 'events', app.programTable, 0, app.timesTable, app.programTimesJoinTable);
                $d.resolve();
            });
            return $d.promise();
        };

        var versionUpdates_2 = function(){
            var $d = $.Deferred();

            $.when(
                cache.executeSql('ALTER TABLE events ADD COLUMN ordering TEXT'),
                cache.executeSql('UPDATE versions SET version = 2')
            ).done(function(){
                $d.resolve();
            });

            return $d.promise();
        };

        //Database versions
        //First version of app, does not have this table so create the table if not exists
        cache.executeSql('CREATE TABLE IF NOT EXISTS versions (id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE, version INTEGER UNIQUE)').done(function(){
            //Version 1 - DB Changes
            cache.fetchColumn('SELECT MAX(version) FROM versions').done(function(version){

                if(!version || version < 1){
                    versionUpdates_1();
                }

                if( version < 2 ){
                    versionUpdates_2();
                }

            });
        });

        //Parse setup
        ParsePushPlugin.subscribe('Users', function(msg) {
        }, function(e) {
        });
    },

    loadTemplate: function(template){
        app.slidingMenu.setMainPage(template, {closeMenu: true});
        app.slidingMenu.setSwipeable(true);
    },

    loadPage: function(title){
        app.pageTitle = title;
        app.loadTemplate('templates/content_page.html');
    },

    openExternalLink: function(url) {
        cordova.InAppBrowser.open(url, '_blank', 'location=yes');
    },

    showLoadingScreen: function(message, $sce, apply) {
        $('ons-sliding-menu').css('opacity', 0.5);

        var $spinContainer = $('.spin-container');

        $spinContainer.css({
            marginLeft: (-1 * ($spinContainer.outerWidth() / 2)),
            marginTop: (-1 * ($spinContainer.outerHeight() / 2))
        });

        app.rootScope.loaderMessage = $sce.trustAsHtml(message);
        app.rootScope.showLoader = true;

        if(apply){
            app.rootScope.$apply();
        }
    },

    hideLoadingScreen: function(){
        $('ons-sliding-menu').css('opacity', 1);

        app.rootScope.showLoader = false;
        app.rootScope.$apply();
    },

    processEventResults: function(results){
        $.each(results, function(key, val){
            results[key].from_date_full = '';
            results[key].to_date_full = '';

            if(val.from_date && val.from_time){
                results[key].from_date_full = val.from_date + 'T' + val.from_time;
            }
            if(val.to_date && val.to_time){
                results[key].to_date_full = val.to_date   + 'T' + val.to_time;
            }

            if('recite_id' in val){
                results[key].hero_image = 'img/events/' + val.recite_id + '.jpg';
            }
        });

        return results;
    },

    internetConnected: function(){
        return navigator.connection.type !== Connection.UNKNOWN && navigator.connection.type !== Connection.NONE;
    }
};

//General controller
module.controller('APAMAppController', function($rootScope, $scope){
    app.rootScope = $rootScope;

    $scope.scrollToTop = function() {
        $('.page__content').animate({scrollTop: 0 });
    };
});

module.controller('ContentPageController', function($scope, $sce) {
    $scope.urlize = function( title ){
        if( typeof title !== 'undefined'){
            return title.toLowerCase().replace(/[^0-9a-z]/g, '-');
        }
        else{
            return '';
        }
    };

    var sql = sprintf('SELECT * FROM %s WHERE title=?', app.contentTable);
    cache.fetchRow(sql, [ app.pageTitle ]).done(function (page) {
        $scope.$apply(function() {
            $scope.title = app.pageTitle;
            $scope.content = $sce.trustAsHtml(page.content);
        });
    });

    cache.needToCheckForUpdates(app.contentTable, 'title', app.pageTitle).done(function(needsCheck, row){
        if(needsCheck){
            var sql = sprintf('SELECT recite_id FROM %s WHERE title = ?', app.contentTable);
            cache.fetchColumn(sql, [ app.pageTitle ]).done(function(recite_id){
                cache.checkAndDownloadUpdatesSingleObject(app.contentUrl, app.contentTable, row.lastChecked, recite_id);
            });
        }
    });
});

module.controller('VenuesController', function($scope, $sce) {
    $scope.data = { venues: [] };

    $scope.loadVenue = function(id) {
        app.venueId = id;
        app.navi.pushPage('templates/venue_details.html');
    };

    cache.loadTable(app.venuesTable, ['sort ASC']).done(function(venues){
        $scope.$apply(function() {
            $scope.data.venues = venues;
        });
    });

    cache.needToCheckForUpdates(app.checksTable, 'entity', 'venues').done(function(needsCheck, row){
        if(needsCheck){
            cache.checkAndDownloadUpdates(app.venuesFeedUrl, 'venues', app.venuesTable, row.lastChecked);
        }
    });
});

module.controller('VenueDetailsController', function($scope) {
    $scope.venue = {};
    $scope.venueImageStyle = {};
    $scope.openExternalLink = app.openExternalLink;
    $scope.hasVenue = false;

    cache.loadSingle(app.venuesTable, app.venueId).done(function(venue){
        $scope.$apply(function() {
            $scope.venue = venue;
            $scope.hasVenue = true;
        });
    });
});

module.controller('DelegatesController', function($scope, $sce) {
    $scope.internet_connected = app.internetConnected();

    if($scope.internet_connected){
        app.showLoadingScreen('Loading...', $sce, false);

        setTimeout(function(){
            app.hideLoadingScreen();
            cordova.InAppBrowser.open(app.delegatesLink, '_system', 'location=yes');
            document.addEventListener("resume", function(){
                location.reload(); //Go back to home page
            }, false);
        },
        3000);
    }
});

module.controller('ProgramController', function($scope, $sce) {
    $scope.dates = [];
    $scope.events = [];
    $scope.genres = [];

    $scope.search = {
        from_date: '',
        genre: ''
    };

    //Tabs
    $scope.tab = 'Schedule';
    $scope.setTab = function(newValue){
        $scope.tab = newValue;
    };
    $scope.isSet = function(tabName){
        return $scope.tab === tabName;
    };
    $scope.filterEvents = function(item){
        //Booths tab
        if($scope.isSet('Booths')) {
            return item.event_type === 'booth';
        }
        //Exchange tab
        else if($scope.isSet('Exchange')){
            return item.event_type === 'exchange';
        }
        //Genres tab
        else if($scope.isSet('Genres')) {
            return item.genre === $scope.search.genre && (item.event_type === 'pitch' || item.event_type === 'showcase' || item.event_type === 'exchange');
        }
        //Schedule tab
        else{
            return item.from_date === $scope.search.from_date && (item.event_type === 'pitch' || item.event_type === 'showcase' || item.event_type === 'exchange');
        }
    };
    $scope.getOrderBy = function(){
        if($scope.isSet('Genres')) {
            return [ 'title' ];
        }
        else if($scope.isSet('Booths')) {
            return [ 'title', 'from_date', 'from_time', 'to_date', 'to_time' ];
        }
        else{
            return  [ 'from_date', 'from_time', 'to_date', 'to_time', 'title' ];
        }
    };

    $scope.loadEvent = function(id) {
        app.eventId = id;
        app.navi.pushPage('templates/event_details.html');
    };


    //Query for dates
    cache.fetchAll('SELECT DISTINCT from_date FROM times ORDER BY from_date').done(function(results){
        var first = true;
        var dates = [];
        var firstDate = '';

        $.each(results, function(k, v){
            dates.push(v.from_date);
            if(first){
                first = false;
                firstDate = v.from_date;
            }
        });

        $scope.$apply(function() {
            $scope.dates = dates;
            $scope.search.from_date = firstDate;
        });
    });

    //Query for genres
    cache.fetchAll("SELECT DISTINCT genre FROM events WHERE genre IS NOT NULL AND genre != '' ORDER BY genre ASC").done(function(results){
        var genres = [];

        $.each(results, function(k, v){
            genres.push(v.genre);
        });
        $scope.$apply(function() {
            $scope.genres = genres;
            //Default to first genre
            $scope.search.genre = genres[0];
        });
    });


    //Query for events
    $scope.ProgramSearch = {
        items: [],
        finished: false
    };

    /*
        SELECT
        events_times.event_id,
        events_times.time_id,
        times.from_date,
        times.from_time,
        times.to_date,
        times.to_time,
        events.recite_id,
        events.title,
        events.duration,
        events.genre,
        events.event_type,
        events.company_name,
        events.venue
        FROM events
        JOIN events_times ON events.id = events_times.event_id
        JOIN times ON times.id = events_times.time_id
        WHERE times.show_in_schedule = '1'
    */
    cache.fetchAll("SELECT events_times.event_id, events_times.time_id, times.from_date, times.from_time, times.to_date, times.to_time, events.recite_id, events.title, events.duration, events.genre, events.event_type, events.company_name, events.venue FROM events JOIN events_times ON events.id = events_times.event_id JOIN times ON times.id = events_times.time_id WHERE times.show_in_schedule = '1'").done(function(results){
        results = app.processEventResults(results);

        $scope.$apply(function() {
            for(var i = 0; i < results.length; i++){
                $scope.ProgramSearch.items.push(results[i]);
            }
            $scope.ProgramSearch.finished = true;
        });
    });

    //Check for updates
    cache.needToCheckForUpdates(app.checksTable, 'entity', 'events')
    .then(function(needsCheck, row){
        if(needsCheck){
            cache.checkAndDownloadUpdates(app.programFeedUrl, 'events', app.programTable, row.lastChecked, app.timesTable, app.programTimesJoinTable);
        }
    });

    //Hide loading spinner initially, then fade in
    $('.loading-results').hide().delay(3000).fadeIn(500);
});

module.controller('EventDetailsController', function($scope, $sce) {
    $scope.event = {};
    $scope.times = [];
    $scope.eventImageStyle = {};
    $scope.hasEvent = false;

    //Tabs
    $scope.tab = 'Event';
    $scope.setTab = function(newValue){
        $scope.tab = newValue;
    };
    $scope.isSet = function(tabName){
        return $scope.tab === tabName;
    };

    //Notes
    $scope.showNotesPage = function() {
        app.navi.pushPage('templates/note.html', {
            event_id: app.eventId,
            noteSavedCallback: function(){
                $scope.hasNote = true;
                $scope.$apply();
            }
        });
    };

    $scope.hasNote = false;
    cache.fetchColumn('SELECT note FROM event_notes WHERE event_id = ?', [ app.eventId ]).done(function(note){
        if(note.length > 0){
            $scope.hasNote = true;
            $scope.$apply();
        }
    });

    //Favourite functionality
    $scope.displayFavouritesButton = false;
    $scope.favouriteActive = false;

    cache.fetchColumn('SELECT count(*) FROM favourite_events WHERE event_id = ?', [ app.eventId ]).done(function(count){
        $scope.favouriteActive = (count > 0);
        $scope.displayFavouritesButton = true;
    });

    $scope.favouritesAction = function(){
        if($scope.favouriteActive){
            //Already favourited, remove favourite
            cache.deleteData('favourite_events', 'event_id', app.eventId).done(function(){
                $scope.favouriteActive = false;
                $scope.$apply();

                ons.notification.alert({
                    title: 'Favourites',
                    message: 'Removed from your favourites'
                });
            });
        }
        else {
            //Add favourite
            cache.insertData('favourite_events', {
                event_id: app.eventId
            }).done(function(id){
                $scope.favouriteActive = true;
                $scope.$apply();

                ons.notification.alert({
                    title: 'Favourites',
                    message: 'Added to your favourites'
                });
            });
        }
    };

    //Query for event
    cache.loadSingle(app.programTable, app.eventId).done(function(event){
        $scope.$apply(function() {
            event.hero_image = 'img/events/' + event.recite_id + '.jpg';

            $scope.event = event;
            $scope.fullDescription = $sce.trustAsHtml(event.full_description);
            $scope.artistDescription = $sce.trustAsHtml(event.artist_description);

            $scope.artistYoutubeVideoUrl = '';
            if(event.artist_youtube_video_id){
                $scope.artistYoutubeVideoUrl = $sce.trustAsResourceUrl('https://www.youtube.com/embed/' + event.artist_youtube_video_id);
            }

            $scope.artistVimeoVideoUrl = '';
            if(event.artist_vimeo_video_id){
                $scope.artistVimeoVideoUrl = $sce.trustAsResourceUrl('https://player.vimeo.com/video/' + event.artist_vimeo_video_id);
            }
        });
    });

    //Query for event times
    cache.loadTable(
        app.timesTable,
        [ 'from_date ASC', 'from_time ASC', 'to_date ASC', 'to_time ASC' ],
        [ 'JOIN events_times ON times.id = events_times.time_id'],
        [ sprintf('event_id = %d', app.eventId) ]
    ).done(function(results){
        $scope.$apply(function() {
            $scope.times = app.processEventResults(results);
        });
    });
});

module.controller('NewsController', function($scope, $sce) {
    $scope.news = [];

    //Tabs
    $scope.tab = 'Updates';
    $scope.setTab = function(newValue){
        $scope.tab = newValue;
    };
    $scope.isSet = function(tabName){
        return $scope.tab === tabName;
    };

    cache.loadTable(app.newsTable, [ 'ts_start DESC' ]).done(function(results){
        $.each(results, function(key, item){
            results[key].body = $sce.trustAsHtml(results[key].body);
            results[key].ts_start = results[key].ts_start * 1000; //convert to ms
            results[key].ts_finish = results[key].ts_finish * 1000; //convert to ms
        });

        $scope.$apply(function() {
            $scope.news = results;
        });
    });

    cache.loadTable(app.notificationsTable, ['lastUpdate DESC']).done(function(results){
        $scope.$apply(function() {
            $scope.notifications = results;
        });
    });

    cache.needToCheckForUpdates(app.checksTable, 'entity', 'notifications').done(function(needsCheck, row){
        if(needsCheck){
            cache.checkAndDownloadUpdates(app.notificationsFeedUrl, 'notifications', app.notificationsTable, row.lastChecked);
        }
    });

    cache.needToCheckForUpdates(app.checksTable, 'entity', 'news').done(function(needsCheck, row){
        if(needsCheck){
            cache.checkAndDownloadUpdates(app.newsFeedUrl, 'news', app.newsTable, row.lastChecked);
        }
    });
});

module.controller('FavouritesController', function($scope){
    $scope.getOrderBy = function(){
        return  [ 'title' ];
    };

    $scope.loadEvent = function(id) {
        if($scope.editMode){
            return;
        }
        app.eventId = id;
        app.navi.pushPage('templates/event_details.html');
    };

    $scope.editMode = false;
    $scope.openEditMode = function(){
        $scope.editMode = true;
    };
    $scope.closeEditMode = function(){
        $scope.editMode = false;
    };

    $scope.deleteFavourite = function(index, id){
        cache.deleteData('favourite_events', 'event_id', id).done(function(){
            $scope.favourites.splice(index, 1);
            $scope.$apply();
        });
    };

    $scope.favourites = [];
    cache.fetchAll('SELECT events.id, events.recite_id, events.title, events.duration, events.genre, events.event_type, events.company_name, events.venue FROM favourite_events JOIN events ON favourite_events.event_id = events.id ORDER BY events.title ASC').done(function(results){
        if(results.length > 0){
            results = app.processEventResults(results);

            $.each(results, function(index, favourite){
                favourite.dayText = '';

                cache.fetchAll('SELECT times.from_date FROM events_times JOIN times ON times.id = events_times.time_id WHERE events_times.event_id = ? AND times.show_in_schedule = 1', [favourite.id]).done(function(times){
                    if(times.length == 1){
                        var days = ["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"];
                        favourite.dayText = days[new Date(times[0].from_date).getDay()];
                    }
                    else if(times.length > 0){
                        favourite.dayText = 'Various days';
                    }

                    $scope.favourites.push(favourite);
                    $scope.$apply();
                });
            });
        }
    });
});

module.controller('NotesController', function($scope, $sce, $http){
    var eventId = app.navi.getCurrentPage().options.event_id;
    var noteSavedCallback = app.navi.getCurrentPage().options.noteSavedCallback;

    $scope.formData = {
        note: ''
    };

    //Load existing note if exists
    cache.fetchColumn('SELECT note FROM event_notes WHERE event_id = ?', [ eventId ]).done(function(note){
        $scope.formData.note = note;
        $scope.$apply();
    });

    $scope.cancel = function(){
        app.navi.popPage();
    };

    $scope.returnToPreviousPage = function() {
        app.navi.popPage({ onTransitionEnd : function() {
            noteSavedCallback();
        }});
    };

    $scope.saveNote = function(){
        cache.dataExists('event_notes', 'event_id', eventId).done(function(exists){
            var data = {
                note: $scope.formData.note,
                event_id: eventId
            };

            //Update existing note
            if(exists){
                cache.updateData('event_notes', data, { event_id: eventId }).done(function(id){
                    $scope.returnToPreviousPage();

                    ons.notification.alert({
                        title: 'Notes',
                        message: 'Note updated'
                    });
                });
            }
            //Insert note into DB
            else{
                cache.insertData('event_notes', data).done(function(id){
                    $scope.returnToPreviousPage();

                    ons.notification.alert({
                        title: 'Notes',
                        message: 'Note saved'
                    });
                });
            }
        });
    };

    $scope.exportNote = function(){
        //Example email message
        /*
        AN EVENING WITH THE MODERN MĀORI QUARTET
        Music to make you melt like golden syrup on a piece of fresh hot fried bread.
        Website Link: http://www.performingartsmarket.com.au/__/apam/event?event_id=15463

        Note:
        Contents of note
        */
        var sql = 'SELECT title, tagline, recite_id FROM events WHERE id = ?';
        cache.fetchRow(sql, [ eventId ]).done(function (event) {
            var longURL = "http://www.performingartsmarket.com.au/__/apam/event?event_id=" + event.recite_id;

            app.showLoadingScreen('Loading...', $sce, false);

            var body = event.title + "\n";
            if(event.tagline.length > 0){
                body += event.tagline + "\n";
            }
            body += "\nWebsite Link: <website-link>\n\nNote:\n" + $scope.formData.note;

            //Shorten URL
            $http.post('https://www.googleapis.com/urlshortener/v1/url?key=AIzaSyDFOq-GJ9EYpFXpXFUsF7zw6M0QI4G__2w', {longUrl: longURL }).then(function(response){
                //Success
                app.hideLoadingScreen();
                window.open('mailto:?&body=' + encodeURIComponent(body.replace('<website-link>', response.data.id)), '_system');

            }, function(error){
                //Error
                app.hideLoadingScreen();
                window.open('mailto:?&body=' + encodeURIComponent(body.replace('<website-link>', longURL)), '_system');
            });
        });
    };
});

module.controller('NotesIndexController', function($scope, $sce){
    $scope.getOrderBy = function(){
        return  [ 'title' ];
    };

    $scope.nl2br = function(str){
        return $sce.trustAsHtml((str + '').replace(/([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1<br />$2'));
    };

    var loadNotes = function(){
        cache.fetchAll('SELECT event_notes.note, events.id, events.recite_id, events.title, events.duration, events.genre, events.event_type, events.company_name, events.venue FROM event_notes JOIN events ON event_notes.event_id = events.id').done(function(results){
            $scope.notes = app.processEventResults(results);
            $scope.$apply();
        });
    };

    $scope.editMode = false;
    $scope.openEditMode = function(){
        $scope.editMode = true;
    };
    $scope.closeEditMode = function(){
        $scope.editMode = false;
    };

    $scope.deleteNote = function(index, id){
        cache.deleteData('event_notes', 'event_id', id).done(function(){
            $scope.notes.splice(index);
            $scope.$apply();
        });
    };

    $scope.showNotesPage = function(eventId) {
        app.navi.pushPage('templates/note.html', {
            event_id: eventId,
            noteSavedCallback: loadNotes
        });
    };
    loadNotes();
});

module.controller('AdvertController', function($scope, $http){
    ons.ready(function() {
        if( app.internetConnected() ){

            ons.createDialog('templates/ad_dialog.html', { parentScope: $scope }).then(function(dialog) {

                $http.get( app.adsFeedUrl + '?ts=' + Math.floor(Date.now() / 1000) )
                    .success(function(data) {
                        $scope.advert_image = data[Math.floor(Math.random()*data.length)].image;
                        $scope.open_external_link = function(){
                            cordova.InAppBrowser.open(data[0].link, '_blank', 'location=yes');
                        };
                        $scope.close_dialog = function(){
                            dialog.hide();
                        };

                        dialog.show();
                    });
            });
        }
    });
});

app.initialize();
